import scrapy
import re
from crawl.comm.football import FTMatch, FTPrice, FTResult, WDLOdds, WDLSpreadOdds, ScoreResult, PointsResult, DoubleResult

class FootballSpider(scrapy.Spider):
  name = 'football'

  def start_requests(self):
    # 受注赛事
    url = 'https://info.sporttery.cn/football/match_list.php'
    yield scrapy.Request(url, self.parseMatch, 'GET')
    
    # 赛果开奖
    yield self.startPareseResult()

  def parseMatch(self, response):
    cssOfMatches = '.all-wrap > .match_list .m-tab tr'

    # 所有赛事
    matches = response.css(cssOfMatches)
    # 当前赛事
    for node in matches:
      tdNodeList = node.css('td')

      matchWeek = tdNodeList[0].css('::text').get() # 赛事编号
      league = tdNodeList[1].css('::text').get()  # 联赛简称
      leagueFullName = tdNodeList[1].attrib.get('title')  # 联赛全称
      leagueName = '|'.join((league if league is not None else '', leagueFullName if leagueFullName is not None else ''))
      homeTeam = tdNodeList[2].css('.zhu::text').get() # 主队
      awayTeam = tdNodeList[2].css('.ke::text').get() # 客队
      matchTime = tdNodeList[3].css('::text').get() # 比赛时间
      saleStatus = tdNodeList[5].css('::text').get() # 销售状态
      saleStatus = saleStatus.strip() if saleStatus is not None else ''
      wdl = self.parsePassWay(tdNodeList[6].css('div'))
      wdls = self.parsePassWay(tdNodeList[7].css('div'))
      score = self.parsePassWay(tdNodeList[8].css('div'))
      points = self.parsePassWay(tdNodeList[9].css('div'))
      double = self.parsePassWay(tdNodeList[10].css('div'))

      # 入库
      self._currentMatch = FTMatch(
        matchWeek,
        matchTime,
        leagueName,
        homeTeam,
        awayTeam,
        saleStatus,
        wdl,
        wdls,
        score,
        points,
        double
      )
      self._currentMatch.persist()

      # 请求比赛赔率
      if saleStatus == '已开售':
        matchLink = tdNodeList[2].css('a').attrib.get('href')
        matchId = re.sub(r'^.*m=', '', matchLink)
        priceAPI = 'https://i.sporttery.cn/api/fb_match_info/get_pool_rs?mid=' + matchId
        yield scrapy.Request(priceAPI, self.parsePrice, 'GET')

  def parsePassWay(self, div):
    # 待开售
    if div is None:
      return 'wait'

    # 地址
    cls = div.attrib.get('class')
    if cls is None:
      return 'wait'

    # 开售单关方式和过关方式
    if "u-dan" in cls:
      return 'pass&single'

    # 仅开售过关方式
    elif "u-cir" in cls:
      return 'pass'

    # 未开售此玩法
    else:
      return 'wait'

  def parsePrice(self, response):
    data = response.json()
    res = data.get('result').get('odds_list')

    ftprice = FTPrice(
      '', # 不需要 matchId
      self._currentMatch.matchWeek,
      self._currentMatch.matchTime,
      self._currentMatch.league,
      self._currentMatch.homeTeam,
      self._currentMatch.awayTeam
    )

    try:
      wdl = res.get('had').get('odds')[-1]
      wdlOdds = WDLOdds(wdl.get('h'), wdl.get('d'), wdl.get('a'))
      ftprice.wdlOdds = wdlOdds
    finally:
      pass

    try:
      wdls = res.get('hhad').get('odds')[-1]
      wdlsOdds = WDLSpreadOdds(wdls.get('h'), wdls.get('d'), wdls.get('a'), wdls.get('goalline'))
      ftprice.wdlsOdds = wdlsOdds
    finally:
      pass

    try:
      score = res.get('crs').get('odds')[-1]
      scoreOdds = ScoreResult(
        score.get('0100'),
        score.get('0200'),
        score.get('0201'),
        score.get('0300'),
        score.get('0301'),
        score.get('0302'),
        score.get('0400'),
        score.get('0401'),
        score.get('0402'),
        score.get('0500'),
        score.get('0501'),
        score.get('0502'),
        score.get('-1-h'),
        score.get('0000'),
        score.get('0101'),
        score.get('0202'),
        score.get('0303'),
        score.get('-1-d'),
        score.get('0001'),
        score.get('0002'),
        score.get('0102'),
        score.get('0003'),
        score.get('0103'),
        score.get('0203'),
        score.get('0004'),
        score.get('0104'),
        score.get('0204'),
        score.get('0005'),
        score.get('0105'),
        score.get('0205'),
        score.get('-1-a')
      )
      ftprice.scoreResult = scoreOdds
    finally:
      pass

    try:
      points = res.get('ttg').get('odds')[-1]
      pointsOdds = PointsResult(
        points.get('s0'),
        points.get('s1'),
        points.get('s2'),
        points.get('s3'),
        points.get('s4'),
        points.get('s5'),
        points.get('s6'),
        points.get('s7')
      )
      ftprice.pointsResult = pointsOdds
    finally:
      pass

    try:
      double = res.get('hafu').get('odds')[-1]
      doubleOdds = DoubleResult(
        double.get('hh'),
        double.get('hd'),
        double.get('ha'),
        double.get('dh'),
        double.get('dd'),
        double.get('da'),
        double.get('ah'),
        double.get('ad'),
        double.get('aa')
      )
      ftprice.doubleResult = doubleOdds
    finally:
      pass

    # 入库
    ftprice.persist()

  def startPareseResult(self):
    url = 'https://info.sporttery.cn/football/match_result.php'
    self._pages = -1
    yield scrapy.Request(url, self.parseResult, 'GET')

    if self._pages > 0:
      for i in range (2, self._pages):
        url = 'https://info.sporttery.cn/football/match_result.php?page=' + str(i)
        yield scrapy.Request(url, self.parseResult, 'GET')

  def parseResult(self, response):
    # 先处理页码
    if self._pages == -1:
      pgNodes = response.css('.m-page .u-pg2')
      if pgNodes is None or len(pgNodes) == 0:
        self._pages = 0
      else:
        self._pages = len(pgNodes) + 1

    # 处理比赛结果
    cssOfMatch = '.all-wrap > .match_list .m-tab tr'
    matches = response.css(cssOfMatch)
    if matches is None: return
    for matchNode in matches:
      tdNodeList = matchNode.css('td')

      matchTime = tdNodeList[0].css('::text').get()
      matchWeek = tdNodeList[1].css('::text').get()
      league = tdNodeList[2].css('::text').get()
      leagueFullName = tdNodeList[2].attrib.get('title')  # 联赛全称
      leagueName = '|'.join((league if league is not None else '', leagueFullName if leagueFullName is not None else ''))
      homeTeam = self.trimBrackets(tdNodeList[3].css('.zhu::text').get()) # 主队
      awayTeam = self.trimBrackets(tdNodeList[3].css('.ke::text').get()) # 客队
      half = tdNodeList[4].css('span::text').get()
      whole = tdNodeList[5].css('span::text').get()
      status = tdNodeList[9].css('span::text').get()

      FTResult(
        matchTime,
        matchWeek,
        leagueName,
        homeTeam,
        awayTeam,
        '', # 不需要单固
        half,
        whole,
        status
      ).persist()

  def trimBrackets(self, str):
    if str is None: return ''

    return re.sub(r'\(.*\)', '', str)
